import { Network } from "@wormhole-foundation/connect-sdk";

// Path for tokens that are acceptable to the Automatic Token Bridge
export const automaticTokensFilePath = (env: Network) => `src/tokens/${env}AutomaticTokens.json`;

// Path for all cached tokens
export const tokenFilePath = (env: Network) => `src/tokens/${env}Tokens.json`;

/**
 * Simple object check.
 * @param item
 * @returns {boolean}
 */
export function isObject(item: any) {
  return item && typeof item === "object" && !Array.isArray(item);
}

/**
 * Deep merge two objects.
 * @param target
 * @param ...sources
 */
export function mergeDeep(target: any, ...sources: any) {
  if (!sources.length) return target;
  const source = sources.shift();

  if (isObject(target) && isObject(source)) {
    for (const key in source) {
      if (isObject(source[key])) {
        if (!target[key]) Object.assign(target, { [key]: {} });
        mergeDeep(target[key], source[key]);
      } else {
        Object.assign(target, { [key]: source[key] });
      }
    }
  }

  return mergeDeep(target, ...sources);
}
